(ns agent-test
  (:require [clojure.java.io :as io])
  (:require [clojure.string :as string])
  (:require [clojure.core.reducers :as r])
  (:require [korma.db :as kdb])
  (:use [korma.core :rename {insert kcinsert}])
  (:require cavm.h2)
  (:require cavm.cgdata)
  (:gen-class))

;(println "OK...")
;(read-line)
;(println "Go")

;(def writer (agent (io/writer "test.out")))
;
;(defn- write [w msg]
;  (println "writing")
;  (.write w msg)
;  w)
;
;(defn- done [w]
;  (println "closing")
;  (.close w)
;  w)
;
;(send-off writer write "hello there \n")
;(send-off writer done)
;(await writer)

;(println "Waiting...")
;(read-line)

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter writer (io/writer outfile))

(doseq [^String line (line-seq reader)]
  (.write writer line)
  (.newLine writer))

(.close reader)
(.close writer))

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter writer (io/writer outfile))

(doseq [line (line-seq reader)]
  (let [in (string/split line #"\t")
        ^String out (string/join "\t" in)]
    (.write writer out))
  (.newLine writer))

(.close reader)
(.close writer))

(defn- parseFloatNA [str]
  (if (or (= str "NA") (= str "nan") (= str ""))
    Double/NaN
    (Float/parseFloat str)))

(defn- parse [line]
  (let [cols (string/split line #"\t")
        values (map parseFloatNA (rest cols))]
    (vec (cons (first cols) values))))

(defn- format-line [cols]
  (let [values (string/join "\t" (map str (rest cols)))]
    (str (first cols) values)))

;
; This is fast. About 30 sec. Uses 1000% cpu.
;
; A straight copy was 17 sec
;

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter writer (io/writer outfile))


(let [lines (rest (line-seq reader))
      out (pmap #(format-line (parse %)) lines)] 
  (doseq [^String line out]
    (.write writer line) 
    (.newLine writer)))

(.close reader)
(.close writer)
(shutdown-agents))

;
; to array & hex
;
; 35 seconds
;
; This is all of the heavy lifting, and the performance is still quite
; high. cpu usage is just under 1000%. Only things left are split into
; files, and get db ids.
;

(def table-writer
  {:encode (fn [scores] (let [s (#'cavm.h2/score-encode scores)] {:scores s :hex (#'cavm.h2/bytes->hex s)}))})

(defn- line->arrays [line]
  (cons (first line) (#'cavm.h2/encode-scores table-writer (rest line))))

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter writer (io/writer outfile))

(defn- format-line-hex [cols]
  (let [values (string/join "\t" (map :hex (rest cols)))]
    (str (first cols) values)))

(let [lines (rest (line-seq reader))
      out (pmap #(format-line-hex (line->arrays (parse %))) lines)] 
  (doseq [^String line out]
    (.write writer line) 
    (.newLine writer)))

(.close reader)
(.close writer)
(shutdown-agents))

;
; split into files
;
; 33 sec. No noticable change in performance.
;

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))
(def probe-file (str outfile ".probes"))
(def score-file (str outfile ".scores"))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter probe-writer (io/writer probe-file))
(def ^java.io.BufferedWriter score-writer (io/writer score-file))

(let [lines (rest (line-seq reader))
      out (pmap #(line->arrays (parse %)) lines)] 
  (doseq [^String line out]
    (doseq [^String col (rest line)]
      (.write score-writer ^String (:hex col))
      (.newLine score-writer))
    (.write probe-writer ^String (first line)) 
    (.newLine probe-writer)))

(.close reader)
(.close score-writer)
(.close probe-writer)
(shutdown-agents))

;
; with db
;
; 37 sec. Slight increase. Is there a way to put the db
; stuff it its own thread? Could use pmap on the sequence. That
; would likely waste more ids, thought that's not a huge problem.
;

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))
(def probe-file (str outfile ".probes"))
(def score-file (str outfile ".scores"))
(def mydb (cavm.h2/create-db (str outfile "-db")))

(kdb/with-db mydb
  (exec-raw "create table foo (id int not null auto_increment primary key)"))

(def ^java.io.BufferedReader reader (io/reader infile))
(def ^java.io.BufferedWriter probe-writer (io/writer probe-file))
(def ^java.io.BufferedWriter score-writer (io/writer score-file))

(kdb/with-db mydb
  (let [id-fn (-> (#'cavm.h2/sequence-for-column "FOO" "ID")
                  (#'cavm.h2/sequence-seq)
                  (#'cavm.h2/seq-iterator))
        lines (rest (line-seq reader))
        out (pmap #(line->arrays (parse %)) lines)] 
    (doseq [^String line out]
      (doseq [^String col (rest line)]
        (.write score-writer (str (id-fn) "\t"))
        (.write score-writer ^String (:hex col))
        (.newLine score-writer))
      (.write probe-writer ^String (first line)) 
      (.newLine probe-writer))))

(.close reader)
(.close score-writer)
(.close probe-writer)
(shutdown-agents))


;
; with cgdata
;

(def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))
(def probe-file (str outfile ".probes"))
(def score-file (str outfile ".scores"))
(def mydb (cavm.h2/create-db (str outfile "-db")))

(kdb/with-db mydb
  (exec-raw "create table foo (id int not null auto_increment primary key)"))

(def ^java.io.BufferedReader reader (io/reader infile))
(def ^java.io.BufferedWriter probe-writer (io/writer probe-file))
(def ^java.io.BufferedWriter score-writer (io/writer score-file))

(kdb/with-db mydb
  (let [id-fn (-> (#'cavm.h2/sequence-for-column "FOO" "ID")
                  (#'cavm.h2/sequence-seq)
                  (#'cavm.h2/seq-iterator))
        lines (rest (line-seq reader))
        out (pmap #(line->arrays (parse %)) lines)] 
    (doseq [^String line out]
      (doseq [^String col (rest line)]
        (.write score-writer (str (id-fn) "\t"))
        (.write score-writer ^String (:hex col))
        (.newLine score-writer))
      (.write probe-writer ^String (first line)) 
      (.newLine probe-writer))))

(.close reader)
(.close score-writer)
(.close probe-writer)
(shutdown-agents)
;
; Using a future.
; 30s
;

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter writer (io/writer outfile))


(let [lines (rest (line-seq reader))
      out (pmap #(format-line (parse %)) lines)] 
  @(future (doseq [^String line out]
    (.write writer line) 
    (.newLine writer))))

(.close reader)
(.close writer)
(shutdown-agents))

;
; With an agent
; 30s
;

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))

(def writer (agent (io/writer outfile)))

(defn- writeln [^java.io.BufferedWriter w ^String msg]
  (.write w msg)
  (.newLine w)
  w)

(defn- closer [^java.io.BufferedWriter w]
  (.close w)
  w)

(let [lines (rest (line-seq reader))
      out (pmap #(format-line (parse %)) lines)] 
  (doseq [^String line out]
    (send-off writer writeln line)))

(.close reader)
(send-off writer closer)
(shutdown-agents))

; pmap
;
; This recovers almost all of the speed

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def writer (io/writer outfile))


(let [lines (line-seq reader)
      out (pmap #(string/join "\t" (string/split % #"\t")) lines)] 
  (doseq [line out]
    (.write writer line) 
    (.newLine writer)))

(.close reader)
(.close writer))

;
; reducers
;
; This is slower than doseq

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter writer (io/writer outfile))

(let [lines (line-seq reader)
      out (r/map #(string/join "\t" (string/split % #"\t")) lines)] 
  (r/reduce (fn [acc ^String line]
              (.write writer line) 
              (.newLine writer)) nil out))

(.close reader)
(.close writer))

;
; reducers again
;
; This consumes all memory, because reduce builds the whole
; seq in memory.

(comment (def infile (nth *command-line-args* 1))
(def outfile (nth *command-line-args* 2))

(def reader (io/reader infile))
(def ^java.io.BufferedWriter writer (io/writer outfile))

(let [lines (line-seq reader)
      out (r/map #(string/join "\t" (string/split % #"\t")) lines)
      out2 (r/reduce conj '() out) ] 
  (doseq [^String line out2]
    (.write writer line) 
    (.newLine writer)))

(.close reader)
(.close writer))

(defn pmap-n [f coll m]
  (let [n (- m 1)
        rets (map #(future (f %)) coll)
        step (fn step [[x & xs :as vs] fs]
               (lazy-seq
                 (if-let [s (seq fs)]
                   (cons (deref x) (step xs (rest s)))
                   (map deref vs))))]
    (step rets (drop n rets))))

(defn seq1 [s]
  (lazy-seq
    (when-let [[x] (seq s)]
      (cons x (seq1 (rest s)))))) 

(comment (take 3 ((fn [f coll]
   (let [n 0
         rets coll
         step (fn step [[x & xs :as vs] fs]
                (lazy-seq
                 (if-let [s (seq fs)]
                   (cons (do (println "<" x ">") x) (step xs (rest s)))
                   (do (println "vs") vs))))]
     (step rets (drop n rets))))
 identity
 (map #(do (println "Realized" %) %) (seq1 (range 40))))))
